/*
 *  $Id: rampfile.c,v 1.4 2003/08/01 12:05:53 luke Exp $
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <stdio.h>
#include <assert.h>
#include <glib.h>

#include "xbr.h"
#include "rampfile.h"
#include "misc.h"

int save_ramps_to_file(char* fn, int ramp_size,
	float brightness_ramp[], float red_ramp[], float green_ramp[], float blue_ramp[]) {
		FILE* f;
	
		if (!(f = fopen(fn, "w"))) {
			/* open failed */
			return -1;
		}
		
		fwrite(&ramp_size, sizeof(ramp_size), 1, f);
		fwrite(brightness_ramp, sizeof(float), ramp_size, f);
		fwrite(red_ramp, sizeof(float), ramp_size, f);
		fwrite(green_ramp, sizeof(float), ramp_size, f);
		fwrite(blue_ramp, sizeof(float), ramp_size, f);
		fclose(f);
		return 0;
}

/* returns TRUE if all values in ramp are in [0,1], FLASE otherwise */
static gboolean check_ramp_range(float ramp[], int size) {
	int i;
	
	for (i = 0; i < size; ++i)
		if (ramp[i] < 0 || ramp[i] > 1)
			return FALSE;
	return TRUE;
}

int read_ramps_from_file(char* fn,
	float brightness_ramp[], float red_ramp[], float green_ramp[], float blue_ramp[]) {
		FILE* f;
		int size, n;
	
		if (!(f = fopen(fn, "r"))) {
			/* cannot open file for reading */
			return -1;
		}
	
		fread(&size, sizeof(size), 1, f);
		if (size != ramp_size) {

			return -3;
		}
			
		n = fread(brightness_ramp, sizeof(float), size, f);
		if (size < n || !check_ramp_range(brightness_ramp, size))  {
			/* file too short or values out of range */
			return -2;
		}
		n = fread(red_ramp, sizeof(float), size, f);
		if (size < n || !check_ramp_range(red_ramp, size))  {
			/* file too short or values out of range */
			return -2;
		}
		n = fread(green_ramp, sizeof(float), size, f);
		if (size < n || !check_ramp_range(green_ramp, size))  {
			/* file too short or values out of range */
			return -2;
		}
		n = fread(blue_ramp, sizeof(float), size, f);
		if (size < n || !check_ramp_range(blue_ramp, size))  {
			/* file too short or values out of range */
			return -2;
		}
		fclose(f);
		return size;
}

int read_ramp(char* fn, float ramp[]) {
	FILE* f;
	int size, n;
	
	if (!(f = fopen(fn, "r"))) {
		return -1;
	}
	
	fread(&size, sizeof(size), 1, f);
	assert(size <= MAX_RAMPSIZE);
	n = fread(ramp, sizeof(ramp[0]), size, f);
	assert(size = n);
	fclose(f);
	return size;
}

void save_ramp(char* fn, int ramp_size, float ramp[]) {
	FILE* f;
	
	if (!(f = fopen(fn, "w"))) 
		return;
		
	fwrite(&ramp_size, sizeof(ramp_size), 1, f);
	fwrite(ramp, sizeof(ramp[0]), ramp_size, f);
	fclose(f);
}
